@@ -1,11 +1,12 @@
 import logging
 import typing
 import discord
+from inspect import iscoroutinefunction
 from discord.ext import commands
 from . import http
 from . import model
+from . import error
 from .utils import manage_commands
-from inspect import iscoroutinefunction
 
 
 class SlashCommand:
@@ -24,21 +25,26 @@ class SlashCommand:
     :ivar req: :class:`.http.SlashCommandRequest` of this client.
     :ivar logger: Logger of this client.
     :ivar auto_register: Whether to register commands automatically.
+    :ivar auto_delete: Whether to delete commands not found in the project automatically.
     :ivar has_listener: Whether discord client has listener add function.
     """
 
     def __init__(self,
                  client: typing.Union[discord.Client, commands.Bot],
                  auto_register: bool = False,
+                 auto_delete: bool = False,
                  override_type: bool = False):
         self._discord = client
         self.commands = {}
         self.subcommands = {}
         self.logger = logging.getLogger("discord_slash")
-        self.req = http.SlashCommandRequest(self.logger)
+        self.req = http.SlashCommandRequest(self.logger, self._discord)
         self.auto_register = auto_register
+        self.auto_delete = auto_delete
         if self.auto_register:
             self._discord.loop.create_task(self.register_all_commands())
+        if self.auto_delete:
+            self._discord.loop.create_task(self.delete_unused_commands())
         if not isinstance(client, commands.Bot) and not isinstance(client,
                                                                    commands.AutoShardedBot) and not override_type:
             self.logger.info("Detected discord.Client! Overriding on_socket_response.")
@@ -76,6 +82,8 @@ class SlashCommand:
         for x in res:
             x.cog = cog
             if isinstance(x, model.CogCommandObject):
+                if x.name in self.commands.keys():
+                    raise error.DuplicateCommand(x.name)
                 self.commands[x.name] = x
             else:
                 if x.base in self.commands.keys():
@@ -84,7 +92,7 @@ class SlashCommand:
                 else:
                     _cmd = {
                         "func": None,
-                        "description": "No description.",
+                        "description": x.base_desc,
                         "auto_convert": {},
                         "guild_ids": x.allowed_guild_ids,
                         "api_options": [],
@@ -96,8 +104,12 @@ class SlashCommand:
                 if x.subcommand_group:
                     if x.subcommand_group not in self.subcommands:
                         self.subcommands[x.base][x.subcommand_group] = {}
+                    if x.name in self.subcommands[x.base][x.subcommand_group].keys():
+                        raise error.DuplicateCommand(f"{x.base} {x.subcommand_group} {x.name}")
                     self.subcommands[x.base][x.subcommand_group][x.name] = x
                 else:
+                    if x.name in self.subcommands[x.base].keys():
+                        raise error.DuplicateCommand(f"{x.base} {x.name}")
                     self.subcommands[x.base][x.name] = x
 
     def remove_cog_commands(self, cog):
@@ -137,17 +149,66 @@ class SlashCommand:
 
     async def register_all_commands(self):
         """
-        Registers all slash commands except subcommands to Discord API.\n
+        Registers all slash commands to Discord API.\n
         If ``auto_register`` is ``True``, then this will be automatically called.
         """
         await self._discord.wait_until_ready()  # In case commands are still not registered to SlashCommand.
         self.logger.info("Registering commands...")
         for x in self.commands.keys():
             selected = self.commands[x]
-            if selected.has_subcommands and not hasattr(selected, "invoke"):
-                # Just in case it has subcommands but also has base command.
-                # More specific, it will skip if it has subcommands and doesn't have base command coroutine.
-                self.logger.debug("Skipping registering subcommands.")
+            if selected.has_subcommands and selected.func:
+                # Registering both subcommand and command with same base name / name
+                # will result in only subcommand being registered,
+                # so we will warn this at registering subcommands.
+                self.logger.warning(f"Detected command name with same subcommand base name! "
+                                    f"This command will only have subcommand: {x}")
+            if selected.has_subcommands:
+                tgt = self.subcommands[x]
+                options = []
+                for y in tgt.keys():
+                    sub = tgt[y]
+                    if isinstance(sub, model.SubcommandObject):
+                        _dict = {
+                            "name": sub.name,
+                            "description": sub.description if sub.description else "No Description.",
+                            "type": 1,
+                            "options": sub.options if sub.options else []
+                        }
+                        options.append(_dict)
+                    else:
+                        base_dict = {
+                            "name": y,
+                            "description": "No Description.",
+                            "type": 2,
+                            "options": []
+                        }
+                        for z in sub.keys():
+                            sub_sub = sub[z]
+                            _dict = {
+                                "name": sub_sub.name,
+                                "description": sub_sub.description if sub_sub.description else "No Description.",
+                                "type": 1,
+                                "options": sub_sub.options if sub_sub.options else []
+                            }
+                            base_dict["options"].append(_dict)
+                            if sub_sub.sub_group_desc:
+                                base_dict["description"] = sub_sub.sub_group_desc
+                        options.append(base_dict)
+                if selected.allowed_guild_ids:
+                    for y in selected.allowed_guild_ids:
+                        await manage_commands.add_slash_command(self._discord.user.id,
+                                                                self._discord.http.token,
+                                                                y,
+                                                                x,
+                                                                selected.description,
+                                                                options)
+                else:
+                    await manage_commands.add_slash_command(self._discord.user.id,
+                                                            self._discord.http.token,
+                                                            None,
+                                                            x,
+                                                            selected.description,
+                                                            options)
                 continue
             if selected.allowed_guild_ids:
                 for y in selected.allowed_guild_ids:
@@ -166,6 +227,45 @@ class SlashCommand:
                                                         selected.options)
         self.logger.info("Completed registering all commands!")
 
+    async def delete_unused_commands(self):
+        """
+        Unregisters all slash commands which are not used by the project to Discord API.\n
+        This might take some time because for every guild the bot is on an API call is made.\n
+        If ``auto_delete`` is ```True``, then this will be automatically called.
+        """
+        await self._discord.wait_until_ready()
+        self.logger.info("Deleting unused commands...")
+        registered_commands = {}
+        global_commands = await manage_commands.get_all_commands(self._discord.user.id,
+                                                                 self._discord.http.token,
+                                                                 None)
+        for cmd in global_commands:
+            registered_commands[cmd["name"]] = {"id": cmd["id"], "guild_id": None}
+
+        for guild in self._discord.guilds:
+            # Since we can only get commands per guild we need to loop through every one
+            try:
+                guild_commands = await manage_commands.get_all_commands(self._discord.user.id,
+                                                                        self._discord.http.token,
+                                                                        guild.id)
+            except error.RequestFailure:
+                # In case a guild has not granted permissions to access commands
+                continue
+
+            for cmd in guild_commands:
+                registered_commands[cmd["name"]] = {"id": cmd["id"], "guild_id": guild.id}
+
+        for x in registered_commands.keys():
+            if x not in self.commands.keys():
+                # Delete command if not found locally
+                selected = registered_commands[x]
+                await manage_commands.remove_slash_command(self._discord.user.id,
+                                                           self._discord.http.token,
+                                                           selected["guild_id"],
+                                                           selected["id"])
+
+        self.logger.info("Completed deleting unused commands!")
+
     def add_slash_command(self,
                           cmd,
                           name: str = None,
@@ -195,9 +295,11 @@ class SlashCommand:
         name = cmd.__name__ if not name else name
         name = name.lower()
         if name in self.commands.keys():
-            tgt = self.subcommands[name]
-            has_subcommands = tgt["has_subcommands"]
-            guild_ids += tgt["guild_ids"]
+            tgt = self.commands[name]
+            if not tgt.has_subcommands:
+                raise error.DuplicateCommand(name)
+            has_subcommands = tgt.has_subcommands
+            guild_ids += tgt.allowed_guild_ids
         _cmd = {
             "func": cmd,
             "description": description if description else "No description.",
@@ -215,8 +317,11 @@ class SlashCommand:
                        subcommand_group=None,
                        name=None,
                        description: str = None,
+                       base_desc: str = None,
+                       sub_group_desc: str = None,
                        auto_convert: dict = None,
-                       guild_ids: typing.List[int] = None):
+                       guild_ids: typing.List[int] = None,
+                       options: list = None):
         """
         Registers subcommand to SlashCommand.
 
@@ -230,10 +335,16 @@ class SlashCommand:
         :type name: str
         :param description: Description of the subcommand. Default ``None``.
         :type description: str
+        :param base_desc: Description of the base command. Default ``None``.
+        :type base_desc: str
+        :param sub_group_desc: Description of the subcommand_group. Default ``None``.
+        :type sub_group_desc: str
         :param auto_convert: Dictionary of how to convert option values. Default ``None``.
         :type auto_convert: dict
         :param guild_ids: List of guild ID of where the command will be used. Default ``None``, which will be global command.
         :type guild_ids: List[int]
+        :param options: Options of the subcommand.
+        :type options: list
         """
         base = base.lower()
         subcommand_group = subcommand_group.lower() if subcommand_group else subcommand_group
@@ -241,7 +352,7 @@ class SlashCommand:
         name = name.lower()
         _cmd = {
             "func": None,
-            "description": "No description.",
+            "description": base_desc,
             "auto_convert": {},
             "guild_ids": guild_ids,
             "api_options": [],
@@ -251,8 +362,11 @@ class SlashCommand:
             "func": cmd,
             "name": name,
             "description": description,
+            "base_desc": base_desc,
+            "sub_group_desc": sub_group_desc,
             "auto_convert": auto_convert,
             "guild_ids": guild_ids,
+            "api_options": options if options else []
         }
         if base not in self.commands.keys():
             self.commands[base] = model.CommandObject(base, _cmd)
@@ -264,8 +378,12 @@ class SlashCommand:
         if subcommand_group:
             if subcommand_group not in self.subcommands[base].keys():
                 self.subcommands[base][subcommand_group] = {}
+            if name in self.subcommands[base][subcommand_group].keys():
+                raise error.DuplicateCommand(f"{base} {subcommand_group} {name}")
             self.subcommands[base][subcommand_group][name] = model.SubcommandObject(_sub, base, name, subcommand_group)
         else:
+            if name in self.subcommands[base].keys():
+                raise error.DuplicateCommand(f"{base} {name}")
             self.subcommands[base][name] = model.SubcommandObject(_sub, base, name)
         self.logger.debug(
             f"Added subcommand `{base} {subcommand_group if subcommand_group else ''} {cmd.__name__ if not name else name}`")
@@ -283,7 +401,7 @@ class SlashCommand:
         All decorator args must be passed as keyword-only args.\n
         1 arg for command coroutine is required for ctx(:class:`.model.SlashContext`),
         and if your slash command has some args, then those args are also required.\n
-        All args are passed in order.
+        All args must be passed as keyword-args.
 
         .. note::
             Role, User, and Channel types are passed as id if you don't set ``auto_convert``, since API doesn't give type of the option for now.\n
@@ -351,11 +469,15 @@ class SlashCommand:
                    subcommand_group=None,
                    name=None,
                    description: str = None,
+                   base_desc: str = None,
+                   sub_group_desc: str = None,
                    auto_convert: dict = None,
-                   guild_ids: typing.List[int] = None):
+                   guild_ids: typing.List[int] = None,
+                   options: typing.List[dict] = None):
         """
         Decorator that registers subcommand.\n
-        Unlike discord.py, you don't need base command.
+        Unlike discord.py, you don't need base command.\n
+        All args must be passed as keyword-args.
 
         Example:
 
@@ -386,14 +508,28 @@ class SlashCommand:
         :type name: str
         :param description: Description of the subcommand. Default ``None``.
         :type description: str
+        :param base_desc: Description of the base command. Default ``None``.
+        :type base_desc: str
+        :param sub_group_desc: Description of the subcommand_group. Default ``None``.
+        :type sub_group_desc: str
         :param auto_convert: Dictionary of how to convert option values. Default ``None``.
         :type auto_convert: dict
         :param guild_ids: List of guild ID of where the command will be used. Default ``None``, which will be global command.
         :type guild_ids: List[int]
+        :param options: Options of the subcommand. This will affect ``auto_convert`` and command data at Discord API. Default ``None``.
+        :type options: List[dict]
         """
 
+        if options:
+            # Overrides original auto_convert.
+            auto_convert = {}
+            for x in options:
+                if x["type"] < 3:
+                    raise Exception("You can't use subcommand or subcommand_group type!")
+                auto_convert[x["name"]] = x["type"]
+
         def wrapper(cmd):
-            self.add_subcommand(cmd, base, subcommand_group, name, description, auto_convert, guild_ids)
+            self.add_subcommand(cmd, base, subcommand_group, name, description, base_desc, sub_group_desc, auto_convert, guild_ids, options)
             return cmd
 
         return wrapper
