@@ -1,3 +1,4 @@
+import typing
 import asyncio
 import aiohttp
 from ..error import RequestFailure
@@ -93,6 +94,51 @@ async def get_all_commands(bot_id,
             return await resp.json()
 
 
+async def remove_all_commands(bot_id,
+                              bot_token,
+                              guild_ids: typing.List[int] = None):
+    """
+    Remove all slash commands.
+
+    :param bot_id: User ID of the bot.
+    :param bot_token: Token of the bot.
+    :param guild_ids: List of the guild ID to remove commands. Pass ``None`` to remove only the global commands.
+    """
+
+    await remove_all_commands_in(bot_id, bot_token, None)
+
+    for x in guild_ids if guild_ids else []:
+        try:
+            await remove_all_commands_in(bot_id, bot_token, x)
+        except RequestFailure:
+            pass
+
+
+async def remove_all_commands_in(bot_id,
+                                 bot_token,
+                                 guild_id=None):
+    """
+    Remove all slash commands in area.
+
+    :param bot_id: User ID of the bot.
+    :param bot_token: Token of the bot.
+    :param guild_id: ID of the guild to remove commands. Pass `None` to remove all global commands.
+    """
+    commands = await get_all_commands(
+        bot_id,
+        bot_token,
+        guild_id
+    )
+
+    for x in commands:
+        await remove_slash_command(
+            bot_id,
+            bot_token,
+            guild_id,
+            x['id']
+        )
+
+
 def create_option(name: str,
                   description: str,
                   option_type: int,
@@ -102,7 +148,7 @@ def create_option(name: str,
     Creates option used for creating slash command.
 
     :param name: Name of the option.
-    :param description: Descrption of the option.
+    :param description: Description of the option.
     :param option_type: Type of the option.
     :param required: Whether this option is required.
     :param choices: Choices of the option. Can be empty.
